@@ -1,6 +1,6 @@
 	plot.sitar <- function(x, opt="dv", labels, apv=FALSE, xfun, yfun, subset=NULL, abc=c(a=0, b=0, c=0), add=FALSE, nlme=FALSE, ...)
 #	plot curves from sitar model
-#	opt: 
+#	opt:
 #		d = fitted distance curve (labels[1] = x, labels[2] = y)
 #		v = fitted velocity curve (labels[3] = y)
 #		e = fitted fixed effects distance curve (labels[1] = x, labels[2] = y)
@@ -8,7 +8,7 @@
 #		u = unadjusted y vs t curves by subject
 #		a = adjusted y vs adjusted t curves by subject
 #
-#		multiple options all plot on same graph 
+#		multiple options all plot on same graph
 #			(though axes not necessarily optimised)
 #
 #		labels are for opt dv - particularly v
@@ -21,7 +21,7 @@
 #
 #		subset is subset of values
 #
-#		abc is a set of labelled sitar parameters for opt dv e.g.
+#		abc is a set of named sitar parameters for opt dv e.g.
 #			abc=list(a=1, b=0.1, c=-0.1)
 #		or a single id level whose abc values are to be used
 #
@@ -29,13 +29,18 @@
 #
 #		nlme TRUE plots model as nlme object
 {
+  xseq <- function(x, nx=101) {
+    rx <- range(x, na.rm=TRUE)
+    seq(rx[1], rx[2], length.out=nx)
+  }
+
 	model <- x
 	rm(x)
 	if (nlme) {
 		mcall <- match.call()[-1]
 		names(mcall)[[1]] <- 'x'
 		do.call('plot.lme', as.list(mcall))
-	} 
+	}
 	else {
 		mcall <- model$call.sitar
 		data <- eval(mcall$data, parent.frame())
@@ -61,28 +66,39 @@
 		if (!"xlab" %in% names(ARG)) {
 			xl <- ifelse(missing(labels), deparse(mcall$x), labels[1])
 			ARG <- c(ARG, list(xlab=xl))
-		} 
+		}
 		else xl <- ARG$xlab
 #	if ylab not specified replace with label or else y name
 		if (!"ylab" %in% names(ARG)) {
 			yl <- ifelse(missing(labels), deparse(mcall$y), labels[2])
 			ARG <- c(ARG, list(ylab=yl))
-		} 
+		}
 		else yl <- ARG$ylab
 
 #	create output list
 		xy <- list()
-			
+
 #	plot fitted distance and velocity curves
 		if (grepl("d", opt) || grepl("v", opt) || apv) {
-			xt <- x[subset]
-			yt <- fitted(model, level=0)[subset]
+			xt <- xseq(x[subset])
+  		newdata <- data.frame(x=xt)
+# if subset, estimate mean values for covariates
+      if (!identical(subset, rep(TRUE, nf))) {
+  			argnames <- names(formals(model$fitnlme))
+  			xtra <- argnames[!argnames %in% c('x', names(fixef(model)))]
+        if (length(xtra) > 0) {
+          df <- update(model, returndata=TRUE)[subset, xtra]
+          xtra <- unlist(lapply(df, mean, na.rm=TRUE))
+     			newdata <- data.frame(newdata, t(xtra))
+        }
+     }
+			yt <- predict(object=model, newdata=newdata, level=0)
 #	adjust for abc
 			if (!missing(abc)) {
 #	if abc is named ensure 3 values match model random effects
 				if (!is.null(names(abc))) {
 					random <- dimnames(ranef(model))[[2]]
-					for (l in letters[1:3]) 
+					for (l in letters[1:3])
 						if (is.na(abc[l]) || !l %in% random) abc[l] <- 0
 				}
 				else
@@ -96,7 +112,7 @@
 				xoffset <- model$xoffset
 				if (is.null(xoffset)) xoffset <- 0
 				if (!is.na(fixef(model)['b'])) xoffset <- xoffset + fixef(model)['b']
-				xt <- (xt - xoffset + abc[['b']]) / exp(abc[['c']]) + xoffset
+				xt <- (xt - xoffset) / exp(abc[['c']]) + xoffset + abc[['b']]
 				yt <- yt + abc[['a']]
 			}
 #	derive cubic smoothing spline curve
@@ -110,24 +126,24 @@
 				add <- TRUE
 			} else
 			if (grepl("d", opt)) {
-				xy <- do.call("y2plot", c(list(x=quote(ss$x), y1=quote(ss$y), add=add, xy=xy), ARG))
+				xy <- do.call("y2plot", c(list(x=ss$x, y1=ss$y, add=add, xy=xy), ARG))
 				add <- TRUE
 			} else
 			if (grepl("v", opt)) {
 				ARG$ylab <- labels[3]
-				xy <- do.call("y2plot", c(list(x=quote(ss$x), y1=quote(ss1$y), add=add, xy=xy), ARG))
+				xy <- do.call("y2plot", c(list(x=ss$x, y1=ss1$y, add=add, xy=xy), ARG))
 				add <- TRUE
 			}
 		}
-		
+
 #	plot fixed effects distance curve
 		if (grepl("e", opt)) {
-			xt <- x[subset]
-			yt <- model$ns$fitted[subset]
+  		xt <- xseq(x[subset])
+      yt <- predict(model$ns, newdata=data.frame(x=xt))
 			if (!missing(xfun)) xt <- xfun(xt)
 			if (!missing(yfun)) yt <- yfun(yt)
 			ox <- order(xt)
-			xy <- do.call("y2plot", c(list(x=quote(xt[ox]), y1=quote(yt[ox]), add=add, xy=xy), ARG))
+			xy <- do.call("y2plot", c(list(x=xt[ox], y1=yt[ox], add=add, xy=xy), ARG))
 			add <- TRUE
 		}
 
@@ -135,19 +151,19 @@
 		if (grepl("u", opt)) {
 			if (!missing(xfun)) x <- xfun(x)
 			if (!missing(yfun)) y <- yfun(y)
-			do.call("mplot", c(list(x=quote(x), y=quote(y), id=quote(id), subset=quote(subset), add=add), ARG))
+  		do.call("mplot", c(list(x=x, y=y, id=id, subset=subset, add=add), ARG))
 			add <- TRUE
 		}
-	
+
 #	plot fitted curves by subject
 		if (grepl("f", opt)) {
 			y <- fitted(model, level=1)
 			if (!missing(xfun)) x <- xfun(x)
 			if (!missing(yfun)) y <- yfun(y)
-			do.call("mplot", c(list(x=quote(x), y=quote(y), id=quote(id), subset=quote(subset), add=add), ARG))
+    	do.call("mplot", c(list(x=x, y=y, id=id, subset=subset, add=add), ARG))
 			add <- TRUE
 		}
-	
+
 #	plot adjusted y vs adjusted t by subject
 		if (grepl("a", opt)) {
 			fred <- summary(model)
@@ -155,7 +171,7 @@
 			y <- fred$y.adj
 			if (!missing(xfun)) x <- xfun(x)
 			if (!missing(yfun)) y <- yfun(y)
-			do.call("mplot", c(list(x=quote(x), y=quote(y), id=quote(id), subset=quote(subset), add=add), ARG))
+    	do.call("mplot", c(list(x=x, y=y, id=id, subset=subset, add=add), ARG))
 			add <- TRUE
 		}
 #	plot vertical line at age of peak velocity
